package com.indigitall.indigitall_flutter_plugin

import android.content.Context
import android.media.midi.MidiManager
import androidx.core.content.ContextCompat
import com.indigitall.android.Indigitall
import com.indigitall.android.callbacks.DeviceCallback
import com.indigitall.android.commons.models.Channel
import com.indigitall.android.commons.models.Error
import com.indigitall.android.customer.CustomerIndigitall
import com.indigitall.android.customer.callbacks.CustomerCallback
import com.indigitall.android.customer.callbacks.CustomerFieldCallback
import com.indigitall.android.customer.models.Customer
import com.indigitall.android.customer.models.CustomerField
import com.indigitall.android.models.Device
import io.flutter.plugin.common.MethodChannel.Result
import org.json.JSONArray
import org.json.JSONException
import org.json.JSONObject

class IndigitallCustomerFlutterPlugin(val context: Context) {

    private val CALLBACK_CUSTOMER = "callbackCustomer"
    private val CALLBACK_CUSTOMER_FIELDS = "callbackCustomerFields"

    fun getCustomer(result: Result) {
        CustomerIndigitall.getCustomer(context, object: CustomerCallback(context) {
            override fun onError(error: Error?) {
                ContextCompat.getMainExecutor(context).execute {
                    result.error(error?.errorCode?.name, error?.errorMessage, null)
                }
            }

            override fun onSuccess(customer: Customer) {
                ContextCompat.getMainExecutor(context).execute {
                    result.success(mapFromCustomer(customer))
                }
            }

        })
    }

    fun getCustomerInformation(arguments: Any?, result: Result) {
        arguments?.let{
            val jsonArray = JSONArray(it as List<*>)
            val customerFields = parseCustomerArray(jsonArray)
            CustomerIndigitall.getCustomerInformation(context, customerFields, object : CustomerFieldCallback(context){
                override fun onError(error: Error?) {
                    ContextCompat.getMainExecutor(context).execute {
                        result.error(error?.errorCode?.name, error?.errorMessage, null)
                    }
                }

                override fun onSuccess(customerField: ArrayList<CustomerField>) {
                    ContextCompat.getMainExecutor(context).execute {
                        result.success(customerFieldToMap(CALLBACK_CUSTOMER_FIELDS, listFromCustomerArray(customerField)))
                    }
                }

            })
        }
    }

    fun link(arguments: Any?, result: Result) {
        arguments?.let{
            val params = JSONObject(it as Map<*, *>)
            var externalId = ""
            var channel = Channel.PUSH
            if (params.has("channel")) channel = getChannel(params.getString("channel"))
            if (params.has("externalId")) externalId = params.getString("externaId")
            Indigitall.logIn(context, externalId, object: DeviceCallback(context){
                override fun onSuccess(p0: Device?) {
                    CustomerIndigitall.linkCustomer(context, channel, object : CustomerCallback(context){
                        override fun onError(error: Error?) {
                            ContextCompat.getMainExecutor(context).execute {
                                result.error(error?.errorCode?.name, error?.errorMessage, null)
                            }
                        }

                        override fun onSuccess(customer: Customer) {
                            ContextCompat.getMainExecutor(context).execute {
                                result.success("")
                            }
                        }

                    })
                }

                override fun onFail(error: Error?) {
                    ContextCompat.getMainExecutor(context).execute {
                        result.error(error?.errorCode?.name, error?.errorMessage, null)
                    }
                }

            })

        }
    }

    fun unlink(arguments: Any?, result: Result) {
        arguments?.let{
            val params = JSONObject(it as Map<*, *>)
            var channel = Channel.PUSH
            if (params.has("channel")) channel = getChannel(params.getString("channel"))
            Indigitall.logOut(context, object: DeviceCallback(context){
                override fun onSuccess(p0: Device?) {
                    CustomerIndigitall.linkCustomer(context, channel, object : CustomerCallback(context){
                        override fun onError(error: Error?) {
                            ContextCompat.getMainExecutor(context).execute {
                                result.error(error?.errorCode?.name, error?.errorMessage, null)
                            }
                        }

                        override fun onSuccess(customer: Customer) {
                            ContextCompat.getMainExecutor(context).execute {
                                result.success("")
                            }
                        }

                    })
                }

                override fun onFail(error: Error?) {
                    ContextCompat.getMainExecutor(context).execute {
                        result.error(error?.errorCode?.name, error?.errorMessage, null)
                    }
                }

            })

        }
    }

    fun assignOrUpdateValueToCustomerFields(arguments: Any?, result: Result) {
        arguments?.let{
            val params = JSONObject(it as Map<*, *>)
            CustomerIndigitall.updateValueToCustomerFields(context, params, object: CustomerCallback(context) {
                override fun onError(error: Error?) {
                    ContextCompat.getMainExecutor(context).execute {
                        result.error(error?.errorCode?.name, error?.errorMessage, null)
                    }
                }

                override fun onSuccess(customer: Customer) {
                    ContextCompat.getMainExecutor(context).execute {
                        result.success(mapFromCustomer(customer))
                    }
                }

            })
        }
    }

    fun deleteValuesFromCustomerFields(arguments: Any?, result: Result) {
        arguments?.let{
            val jsonArray = JSONArray(it as List<*>)
            CustomerIndigitall.deleteValuesFromCustomerFields(context, parseCustomerArray(jsonArray), object: CustomerCallback(context) {
                override fun onError(error: Error?) {
                    ContextCompat.getMainExecutor(context).execute {
                        result.error(error?.errorCode?.name, error?.errorMessage, null)
                    }
                }

                override fun onSuccess(customer: Customer) {
                    ContextCompat.getMainExecutor(context).execute {
                        result.success(mapFromCustomer(customer))
                    }
                }

            })
        }
    }


    private fun getChannel(channel: String): Channel {
        return when (channel) {
            "inapp" -> {
                Channel.INAPP
            }
            "chat" -> {
                Channel.CHAT
            }
            else -> Channel.PUSH
        }
    }

    private fun mapFromCustomer(customer: Customer): Map<String, Map<*,*>> {
        val MAP_ID = "id"
        val MAP_CUSTOMER_ID = "customerId"
        val MAP_APPLICATION_ID = "applicationId"
        val MAP_CREATED_AT = "createdAt"
        val MAP_UPDATED_AT = "updatedAt"

        val map: MutableMap<String, String> = HashMap()
        try {
            customer.id?.let { map[MAP_ID] = it.toString() }
            customer.customerId?.let { map[MAP_CUSTOMER_ID] = it.toString() }
            customer.createdAt?.let { map[MAP_CREATED_AT] = it.toString() }
            customer.applicationId?.let { map[MAP_APPLICATION_ID] = it.toString() }
            customer.updatedAt?.let { map[MAP_UPDATED_AT] = it.toString() }

        } catch (ex: Exception) {
            ex.printStackTrace()
        }
        val customerMap: MutableMap<String, Map<*,*>> = HashMap()
        customerMap[CALLBACK_CUSTOMER] = map
        return customerMap
    }

    private fun listFromCustomerArray(customerFields: ArrayList<CustomerField>): ArrayList<Map<*, *>> {
        val list = arrayListOf<Map<*, *>>()
        for (i in customerFields.indices){
            list.add(mapFromCustomerFields(customerFields[i]))
        }
        return list
    }

    private fun mapFromCustomerFields(field: CustomerField): Map<*, *> {
        val map : MutableMap<String, String> = HashMap()
        field.customerFieldKey?.let{
            if (field.customerFieldValue != null){
                map[it] = field.customerFieldValue!!
            }else{
                map[it] = ""
            }
        }
        return map
    }

    private fun customerFieldToMap(keyString: String, value: ArrayList<Map<*, *>>): Map<*, *>{
        val map: MutableMap<String, ArrayList<Map<*, *>>> = HashMap()
        map[keyString] = value
        return map
    }

    private fun parseCustomerArray(params: JSONArray?): ArrayList<String> {
        var fields = arrayListOf<String>()
        try {
            params?.let{
                if (it.length() > 0) {
                    for (i in 0 until it.length()) {
                        fields.add(it.getString(i))
                    }
                }
            }
        } catch (ex: JSONException) {
            ex.printStackTrace()
        }
        return fields
    }
}


