package com.indigitall.indigitall_flutter_plugin

import android.Manifest
import android.app.Activity
import android.content.Context
import android.content.Intent
import android.content.pm.PackageManager
import android.graphics.BitmapFactory
import android.graphics.Color
import androidx.annotation.NonNull

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import org.json.JSONException
import org.json.JSONObject

import com.indigitall.android.Indigitall
import com.indigitall.android.Configuration

import org.json.JSONArray

import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.view.View
import android.webkit.WebView
import android.widget.ImageButton
import androidx.core.app.ActivityCompat
import androidx.core.content.ContextCompat
import com.google.firebase.messaging.RemoteMessage
import com.indigitall.android.callbacks.*
import com.indigitall.android.commons.models.*
import com.indigitall.android.commons.utils.ErrorUtils
import com.indigitall.android.commons.utils.ErrorUtils.pushError
import com.indigitall.android.commons.utils.Log
import com.indigitall.android.commons.utils.PreferenceUtils
import com.indigitall.android.inapp.InAppIndigitall
import com.indigitall.android.inapp.Utils.*
import com.indigitall.android.inapp.callbacks.InAppWasShownCallback
import com.indigitall.android.inapp.callbacks.ShowInAppCallback
import com.indigitall.android.inapp.models.*
import com.indigitall.android.inbox.Inbox
import com.indigitall.android.inbox.callbacks.InboxCallback
import com.indigitall.android.models.*
import com.indigitall.android.utils.FirebaseUtils

import com.indigitall.android.utils.ServiceUtils
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.PluginRegistry
import java.lang.Exception
import java.net.URL
import java.net.URLDecoder
import java.net.URLEncoder
import java.util.*
import kotlin.collections.ArrayList
import kotlin.collections.HashMap


/** IndigitallFlutterPlugin */
class IndigitallFlutterPlugin: FlutterPlugin, MethodCallHandler, PluginRegistry.RequestPermissionsResultListener, ActivityAware {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private lateinit var channel : MethodChannel

  private val ACTION_INIT = "initialize"
  private val ACTION_DEVICE_ENABLE = "deviceEnable"
  private val ACTION_DEVICE_DISABLE = "deviceDisable"
  private val ACTION_TOPICS_LIST = "topicsList"
  private val ACTION_TOPICS_SUBSCRIBE = "topicsSubscribe"
  private val ACTION_TOPICS_UNSUBSCRIBE = "topicsUnsubscribe"
  private val ACTION_EXTERNAL_CODE = "setExternalCode"
  private val ACTION_SHOW_INAPP = "showInApp"
  private val ACTION_SHOW_MULTIPLE_INAPP = "showMultipleInApp"
  private val ACTION_SHOW_POPUP = "showPopUp"
  private val ACTION_SEND_CUSTOM_EVENT = "sendCustomEvent"
  private val ACTION_GET_PACKAGE_NAME = "packageName"
  private val REQUEST_CODE_ASK_PERMISSIONS = 50001
  private val ACTION_GET_PUSH = "getPush"
  private val ACTION_LOGIN = "logIn"
  private val ACTION_LOGOUT = "logOut"
  private val ACTION_GET_INAPP = "getInApp"
  private val ACTION_POPUP = "popUpAction"
  private val ACTION_GET_POPUP_SCALE = "popUpScale"
  private val ACTION_TAP_INAPP = "tapInApp"
  private val ACTION_GET_INBOX = "getInbox"
  private val ACTION_GET_MESSAGE_COUNT = "getMessageCount"
  private val ACTION_GET_NEXT_PAGE = "getNextPage"
  private val ACTION_MASSIVE_EDIT = "massiveEditNotificationsWithSendingIdsList"
  private val ACTION_GET_INFO_NOTIFICATION = "getInfoFromNotificationWithSendingId"
  private val ACTION_MODIFY_STATUS = "modifyStatusFromNotificationWithSendingId"
  private val ACTION_GET_AUTH_CONFIG = "getAuthConfig"
  private val ACTION_IS_INDIGITALL_PUSH_NOTIFICATION = "isIndigitallPushNotification"
  private val ACTION_GET_CUSTOMER = "getCustomer"
  private val ACTION_GET_CUSTOMER_INFORMATION = "getCustomerInformation"
  private val ACTION_UPDATE_VALUE_CUSTOMER_FIELDS = "assignOrUpdateValueToCustomerFields"
  private val ACTION_DELETE_VALUES_CUSTOMER_FIELDS = "deleteValuesFromCustomerFields"
  private val ACTION_LINK = "link"
  private val ACTION_UNLINK = "unlink"
  private val ACTION_ADD_NEW_INAPP_TO_DISMISS_FOREVER = "addNewInAppToDismissForever"

  private val CALLBACK_ON_INDIGITALL_INITIALIZED = "callbackOnIndigitallInitialized"
  private val CALLBACK_ON_NEW_USER_REGISTERED = "callbackOnNewUserRegistered"
  private val CALLBACK_DEVICE_ENABLE = "callbackDeviceEnabled"
  private val CALLBACK_DEVICE_DISABLE = "callbackDeviceDisEnabled"
  private val CALLBACK_TOPIC_LIST = "callbackTopicList"
  private val CALLBACK_TOPIC_SUBSCRIBE = "callbackTopicSubscribe"
  private val CALLBACK_TOPIC_UNSUBSCRIBE = "callbackTopicUnsubscribe"
  private val CALLBACK_EXTERNAL_CODE = "callbackExternalCode"
  private val CALLBACK_SEND_CUSTOM_EVENT = "callbackSendCustomEvent"
  private val CALLBACK_LOGIN = "callbackLogin"
  private val CALLBACK_LOGOUT = "callbackLogout"
  private val CALLBACK_GET_PUSH = "callbackgetPush"
  private val CALLBACK_GET_INAPP = "callbackGetInApp"
  private val CALLBACK_DID_DISMISSED = "callbackDidDismissed"
  private val CALLBACK_DID_CLOSED = "callbackDidClosed"
  private val CALLBACK_DID_CLICKED = "callbackDidClicked"
  private val CALLBACK_SHOW_POPUP = "callbackShowPopUp"
  private val CALLBACK_SHOWTIME_FINISHED = "callbackShowTimeFinished"
  private val CALLBACK_ON_ERROR = "callbackOnError"

  private var context: Context?= null
  private var activityName = ""
  private var activity: Activity?= null

  private val ACTION_ERROR = "error"
  private val ACTION_SHOWTIME_FINISHED = "showTimeFinished"

  private val INAPP_CODE = "inAppCode"
  private val WEBVIEW = "webView"
  private val MESSAGE = "message"
  private val SHOWTIME = "showtime"
  private val INAPP = "inApp"
  private val WIDTH = "width"
  private val HEIGHT = "height"

  private val VIEW = "view"
  private val POP_UP_CODE = "popUpCode"
  private val CLOSE_BUTTON = "closeButton"
  private val CLOSE_ICON_DISABLED = "closeIconDisabled"

  private var indigitallFlutterInbox: Inbox? = null

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    channel = MethodChannel(flutterPluginBinding.binaryMessenger, "indigitall_flutter_plugin")
    channel.setMethodCallHandler(this)
    context = flutterPluginBinding.applicationContext
  }

  override fun onAttachedToActivity(binding: ActivityPluginBinding) {
    binding.addRequestPermissionsResultListener(this)
    activity = binding.activity
    activityName = activity?.javaClass?.name ?: ""
  }

  override fun onDetachedFromActivityForConfigChanges() {
    activity = null
  }

  override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    binding.addRequestPermissionsResultListener(this)
    activity = binding.activity
    activityName = activity?.javaClass?.name ?: ""
  }

  override fun onDetachedFromActivity() {
    activity = null
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {

    when (call.method) {
      "getPlatformVersion" -> {
        result.success("Android ${android.os.Build.VERSION.RELEASE}")
      }
      ACTION_INIT -> {
        initialize(call.arguments, result)
      }
      ACTION_DEVICE_ENABLE -> {
        deviceEnable(result)
      }
      ACTION_DEVICE_DISABLE -> {
        deviceDisable(result)
      }
      ACTION_TOPICS_LIST -> {
        topicsList(result)
      }
      ACTION_TOPICS_SUBSCRIBE -> {
        topicsSubscribe(call.arguments, result)
      }
      ACTION_TOPICS_UNSUBSCRIBE -> {
        topicsUnsubscribe(call.arguments, result)
      }
      ACTION_EXTERNAL_CODE -> {
        setExternalCode(call.arguments, result)
      }
      ACTION_GET_INAPP -> {
        getInApp(call.arguments, result)
      }
      ACTION_SHOW_POPUP -> {
        showPopUp(call.arguments, result)
      }
      ACTION_TAP_INAPP -> {
        tapInApp(call.arguments)
      }
      ACTION_ADD_NEW_INAPP_TO_DISMISS_FOREVER -> {
        context?.let{
          addNewInAppToDismissForever(call.arguments)
        }
      }
      ACTION_SEND_CUSTOM_EVENT -> {
        sendCustomEvent(call.arguments, result)
      }
      ACTION_GET_PACKAGE_NAME -> {
        getPackageName(result)
      }
      ACTION_LOGIN -> {
        logIn(call.arguments, result)
      }
      ACTION_LOGOUT -> {
        logOut(result)
      }
      ACTION_IS_INDIGITALL_PUSH_NOTIFICATION -> {
        isIndigitallPushNotification(call.arguments, result)
      }
      ACTION_GET_PUSH -> {
        getPush(result)
      }
      ACTION_GET_INBOX -> {
        context?.let {
          IndigitallInboxFlutterPlugin(it).getInbox(call.arguments, object : InboxCallback(it) {
            override fun onError(error: Error) {}

            override fun onSuccess(inbox: Inbox) {
              indigitallFlutterInbox = inbox
            }
          }, result)
        }
      }
      ACTION_GET_MESSAGE_COUNT -> {
        IndigitallInboxFlutterPlugin(context).getMessagesCount(call.arguments, result)
      }
      ACTION_GET_NEXT_PAGE -> {
        IndigitallInboxFlutterPlugin(context).getNextPage(indigitallFlutterInbox, call.arguments, result)
      }
      ACTION_MASSIVE_EDIT -> {
        IndigitallInboxFlutterPlugin(context).massiveEdit(indigitallFlutterInbox, call.arguments, result)
      }
      ACTION_GET_INFO_NOTIFICATION -> {
        IndigitallInboxFlutterPlugin(context).getInfoFromNotification(indigitallFlutterInbox, call.arguments, result)
      }
      ACTION_MODIFY_STATUS -> {
        IndigitallInboxFlutterPlugin(context).modifyStatus(indigitallFlutterInbox, call.arguments, result)
      }
//      ACTION_GET_AUTH_CONFIG -> {
//        call.arguments?.let{
//          IndigitallInboxFlutterPlugin(context).jsonAuth = JSONObject(it as Map<*, *>)
//        }
//      }

      ACTION_GET_CUSTOMER -> {
        context?.let{
          IndigitallCustomerFlutterPlugin(it).getCustomer(result)
        }
      }
      ACTION_GET_CUSTOMER_INFORMATION -> {
        context?.let{
          IndigitallCustomerFlutterPlugin(it).getCustomerInformation(call.arguments, result)
        }
      }
      ACTION_LINK -> {
        context?.let{
          IndigitallCustomerFlutterPlugin(it).link(call.arguments, result)
        }
      }
      ACTION_UNLINK -> {
        context?.let{
          IndigitallCustomerFlutterPlugin(it).unlink(call.arguments, result)
        }
      }
      ACTION_UPDATE_VALUE_CUSTOMER_FIELDS -> {
        context?.let{
          IndigitallCustomerFlutterPlugin(it).assignOrUpdateValueToCustomerFields(call.arguments, result)
        }
      }
      ACTION_DELETE_VALUES_CUSTOMER_FIELDS -> {
        context?.let{
          IndigitallCustomerFlutterPlugin(it).deleteValuesFromCustomerFields(call.arguments, result)
        }
      }
      else -> {
        result.notImplemented()
      }
    }
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    channel.setMethodCallHandler(null)
  }

  private fun initialize(arguments: Any?, result: Result) {
    val params = JSONObject(arguments as Map<*, *>)

    val MAP_APP_KEY = "appKey"
    val MAP_SENDER_ID = "senderId"
    val MAP_REQUEST_LOCATION = "requestLocation"
    val MAP_PRODUCT_NAME = "productName"
    val MAP_PRODUCT_VERSION = "productVersion"
    val MAP_WIFI_FILTER = "wifiFilterEnabled"
    val MAP_URL_DEVICE_API = "urlDeviceApi"
    val MAP_URL_INAPP_API = "urlInAppApi"
    val MAP_URL_INBOX_API = "urlInboxApi"

    var appKey: String? = null
    var senderId: String? = null
    var productName: String? = null
    var productVersion: String? = null
    var urlDeviceApi = "https://device-api.indigitall.com/v1"
    var urlInAppApi = "https://inapp-api.indigitall.com/v1"
    var urlInboxApi = "https://inbox-api.indigitall.com/v1"
    var requestLocation = false
    var wifiFilterEnabled = false

    try {
      params?.let{obj ->
        if (obj.has(MAP_APP_KEY)) appKey = obj.getString(MAP_APP_KEY)
        if (obj.has(MAP_SENDER_ID)) senderId = obj.getString(MAP_SENDER_ID)
        if (obj.has(MAP_REQUEST_LOCATION)) requestLocation = obj.getBoolean(MAP_REQUEST_LOCATION)
        if (obj.has(MAP_PRODUCT_NAME)) productName = obj.getString(MAP_PRODUCT_NAME)
        if (obj.has(MAP_PRODUCT_VERSION)) productVersion = obj.getString(MAP_PRODUCT_VERSION)
        if (obj.has(MAP_WIFI_FILTER)) wifiFilterEnabled = obj.getBoolean(MAP_WIFI_FILTER)
        if (obj.has(MAP_URL_DEVICE_API)) urlDeviceApi = obj.getString(MAP_URL_DEVICE_API)
        if (obj.has(MAP_URL_INAPP_API)) urlInAppApi = obj.getString(MAP_URL_INAPP_API)
        if (obj.has(MAP_URL_INBOX_API)) urlInboxApi = obj.getString(MAP_URL_INBOX_API)
      }
    } catch (ex: JSONException) {
      ex.printStackTrace()
    }

    if (requestLocation){
      if (context?.let { ContextCompat.checkSelfPermission(it, Manifest.permission.ACCESS_FINE_LOCATION) } != PackageManager.PERMISSION_GRANTED){
        activity?.let { ActivityCompat.requestPermissions(it, arrayOf(Manifest.permission.ACCESS_FINE_LOCATION), REQUEST_CODE_ASK_PERMISSIONS) }
      }
    }

    if (appKey != null && senderId != null) {
      val config: Configuration = Configuration.Builder(appKey, senderId)
        .setDefaultActivity(activityName)
        .setAutoRequestPermissionLocation(requestLocation)
        .setProductName(productName)
        .setProductVersion(productVersion)
        .wifiFilterEnabled(wifiFilterEnabled)
        .setUrlDeviceApi(urlDeviceApi)
        .setUrlInAppApi(urlInAppApi)
        .setUrlInboxApi(urlInboxApi)
        .build()
      //llamada init
      Indigitall.init(context, config, object : InitCallBack(context) {
        override fun onIndigitallInitialized(permissions: Array<Permission?>?, device: Device?) {
          ContextCompat.getMainExecutor(context).execute {
            result.success(device?.let { mapFromDevice(it, CALLBACK_ON_INDIGITALL_INITIALIZED)})
          }
        }

        override fun onNewUserRegistered(device: Device?) {
          ContextCompat.getMainExecutor(context).execute {
            result.success(device?.let { mapFromDevice(it, CALLBACK_ON_NEW_USER_REGISTERED) })
          }
        }

        override fun onErrorInitialized(s: String) {
          ContextCompat.getMainExecutor(context).execute {
            result.error("onErrorInitialized", s, null)
          }
        }
      })
    }
  }

  private fun getPackageName(result: Result) {
    context?.let {
      ContextCompat.getMainExecutor(context).execute {
        result.success(stringToMap("packageName", it.packageName))
      }
    }
  }

  private fun deviceEnable(result: Result) {
    Indigitall.deviceEnable(context, object : DeviceCallback(context) {
      override fun onSuccess(device: Device?) {
        ContextCompat.getMainExecutor(context).execute {
          result.success(device?.let { mapFromDevice(it, CALLBACK_DEVICE_ENABLE) })
        }
      }

      override fun onFail(error: Error) {
        ContextCompat.getMainExecutor(context).execute {
          result.error(error.errorCode?.name, error.errorMessage, null)
        }
      }
    })
  }

  private fun deviceDisable(result: Result) {
    Indigitall.deviceDisable(context, object : DeviceCallback(context) {
      override fun onSuccess(device: Device?) {
        ContextCompat.getMainExecutor(context).execute {
          result.success(device?.let { mapFromDevice(it, CALLBACK_DEVICE_DISABLE) })
        }
      }

      override fun onFail(error: Error) {
        ContextCompat.getMainExecutor(context).execute {
          result.error(error.errorCode?.name, error.errorMessage, null)
        }
      }
    })
  }

  private fun topicsList(result: Result) {
    Indigitall.topicsList(context, object : TopicsCallback() {
      override fun onSuccess(topics: Array<Topic>) {
        ContextCompat.getMainExecutor(context).execute {
          result.success(topicToMap(CALLBACK_TOPIC_LIST, listFromTopicArray(topics)))
        }
      }

      override fun onFail(error: Error) {
        ContextCompat.getMainExecutor(context).execute {
          result.error(error.errorCode?.name, error.errorMessage, null)
        }
      }
    })
  }

  private fun topicsSubscribe(
    arguments: Any?,
    result: Result
  ) {
    arguments?.let{
      val jsonArray = JSONArray(it as List<*>)
      val topics = parseTopicArray(jsonArray)
      Indigitall.topicsSubscribe(context, topics, object : TopicsCallback() {
        override fun onSuccess(topics: Array<Topic>) {
          ContextCompat.getMainExecutor(context).execute {
            result.success(topicToMap(CALLBACK_TOPIC_SUBSCRIBE, listFromTopicArray(topics)))
          }
        }

        override fun onFail(error: Error) {
          ContextCompat.getMainExecutor(context).execute {
            result.error(error.errorCode?.name, error.errorMessage, null)
          }
        }
      })
    }
  }

  private fun topicsUnsubscribe(
    arguments: Any?,
    result: Result
  ) {
    arguments?.let{
      val jsonArray = JSONArray(it as List<*>)
      val topics = parseTopicArray(jsonArray)
      Indigitall.topicsUnsubscribe(context, topics, object : TopicsCallback() {
        override fun onSuccess(topics: Array<Topic>) {
          ContextCompat.getMainExecutor(context).execute {
            result.success(topicToMap(CALLBACK_TOPIC_UNSUBSCRIBE, listFromTopicArray(topics)))
          }
        }

        override fun onFail(error: Error) {
          ContextCompat.getMainExecutor(context).execute {
            result.error(error.errorCode?.name, error.errorMessage, null)
          }
        }
      })
    }
  }

  @Throws(JSONException::class)
  private fun setExternalCode(
    arguments: Any?,
    result: Result
  ) {
    arguments?.let {
      val params = arguments as String
      Indigitall.setExternalCode(
        context,
        params,
        object : DeviceCallback(context) {
          override fun onSuccess(device: Device?) {
            ContextCompat.getMainExecutor(context).execute {
              result.success(device?.let { mapFromDevice(it, CALLBACK_EXTERNAL_CODE) })
            }
          }

          override fun onFail(error: Error) {
            ContextCompat.getMainExecutor(context).execute {
              result.error(error.errorCode?.name, error.errorMessage, null)
            }
          }
        })
    }
  }

  private fun isIndigitallPushNotification(arguments: Any?, result: Result) {
    arguments?.let{
      val thread = Thread {
        try {
          val json = JSONObject(arguments as Map<*, *>)
          try {

            val push = Push(context, json)

            if (push?.appKey != null && push.appKey == PreferenceUtils.getAppKey(context)) {
              PushNotification(push).showNotification(context)

              ContextCompat.getMainExecutor(context).execute {
                result.success(true)
              }
            }else{
              ContextCompat.getMainExecutor(context).execute {
                result.success(false)
              }
            }
          } catch (ex: JSONException) {
            ex.printStackTrace()
            ContextCompat.getMainExecutor(context).execute {
              result.success(false)
            }
          }
        } catch (e: Exception) {
          e.printStackTrace()
        }
      }
      thread.start()

//

    }
  }

  @Throws(JSONException::class)
  private fun getPush(result: Result) {
    activity?.intent?.let{
      it.extras?.let {itExtras ->
        val extras: Bundle = itExtras
        if (extras.containsKey(Push.EXTRA_PUSH)) {
          var push: Push? = null
          push = if (context != null) {
            Push(context, extras.getString(Push.EXTRA_PUSH))
          } else {
            Push(extras.getString(Push.EXTRA_PUSH))
          }
          ContextCompat.getMainExecutor(context).execute {
            result.success(mapFromPush(push))
          }
        }
      }
    }
  }


  @Throws(JSONException::class)
  private fun sendCustomEvent(
    arguments: Any?,
    result: Result
  ) {
    arguments?.let{
      val obj = JSONObject(arguments as Map<*, *>)
      val MAP_EVENT = "event"
      val MAP_EVENT_DATA = "customData"
      var eventCustom: String? = null
      var eventCustomData: JSONObject ?= null

      if (obj.has(MAP_EVENT)) eventCustom = obj.getString(MAP_EVENT)
      if (obj.has(MAP_EVENT_DATA)) eventCustomData = JSONObject(obj.getString(MAP_EVENT_DATA))

      context?.let {
        Indigitall.sendCustomEvent(
          it,
          eventCustom,
          eventCustomData,
          object : EventCallback(it) {
            override fun onSuccess() {
              ContextCompat.getMainExecutor(context).execute {
                result.success(stringToMap(CALLBACK_SEND_CUSTOM_EVENT, "event success"))
              }
            }

            override fun onError(error: Error) {
              ContextCompat.getMainExecutor(context).execute {
                result.error(error.errorCode?.name, error.errorMessage, null)
              }
            }
          })
      }
    }

  }


  @Throws(JSONException::class)
  private fun logIn(arguments: Any?, result: Result) {
    arguments?.let{
      val params = it as String
      Indigitall.logIn(context, params, object : DeviceCallback(context) {
        override fun onSuccess(device: Device?) {
          ContextCompat.getMainExecutor(context).execute {
            result.success(device?.let { mapFromDevice(it, CALLBACK_LOGIN) })
          }
        }

        override fun onFail(error: Error) {
          ContextCompat.getMainExecutor(context).execute {
            result.error(error.errorCode?.name, error.errorMessage, null)
          }
        }
      })
    }
  }

  @Throws(JSONException::class)
  private fun logOut(result: Result) {
    Indigitall.logOut(context, object : DeviceCallback(context) {
      override fun onSuccess(device: Device?) {
        ContextCompat.getMainExecutor(context).execute {
          result.success(device?.let { mapFromDevice(it, CALLBACK_LOGOUT) })
        }
      }

      override fun onFail(error: Error) {
        ContextCompat.getMainExecutor(context).execute {
          result.error(error.errorCode?.name, error.errorMessage, null)
        }
      }
    })
  }

  private fun getInApp(arguments: Any?, result: Result){
    arguments?.let{it1 ->
      val inAppId = it1 as String
      context?.let {
        InAppIndigitall.inAppGet(it, inAppId, object : com.indigitall.android.inapp.callbacks.InAppCallback(it) {
          override fun onSuccess(inApp: InApp?) {
            inApp?.let{itInApp ->
              if (itInApp.properties != null) {
                if (itInApp.properties.dismissForever &&
                  PopUpUtils.isInAppDismissForever(it, itInApp)) {
                  ContextCompat.getMainExecutor(context).execute {
                    result.error("2304", "InApp was dismissed forever", null)
                  }
                } else {
                  InAppUtils.inAppWasShown(it, itInApp, object : InAppWasShownCallback(){
                    override fun didExpired() {
                      ContextCompat.getMainExecutor(context).execute {
                        result.error("2101", "InApp was expired", null)
                      }
                    }

                    override fun didShowMoreThanOnce() {
                      ContextCompat.getMainExecutor(context).execute {
                        result.error(
                          "2102",
                          "InApp was shown more than " + inApp.properties.numberOfShows + " times",
                          null
                        )
                      }
                    }

                    override fun didClickOut() {
                      ContextCompat.getMainExecutor(context).execute {
                        result.error(
                          "2103",
                          "InApp was clicked more than " + inApp.properties.numberOfClicks + " times",
                          null
                        )
                      }
                    }

                    override fun onSuccess() {
                      ContextCompat.getMainExecutor(context).execute {
                        result.success(inAppToMap(itInApp))
                      }
                    }
                  })
                }
              } else {
                ContextCompat.getMainExecutor(context).execute {
                  result.success(inAppToMap(itInApp))
                }
              }
            }
          }

          override fun onFail(error: Error?) {
            super.onFail(error)
            ContextCompat.getMainExecutor(context).execute {
              result.error(error?.errorCode?.name, error?.errorMessage, null)
            }
          }
        })
      }
    }
  }

  private fun tapInApp(arguments: Any?) {
    try {
      val obj = JSONObject(arguments as Map<*, *>)
      var inApp = InApp(obj)
      if (inApp.properties != null && inApp.properties.numberOfClicks > 0) context?.let {
        InAppTimesClickedUtils.addInAppNewClick(
          it, inApp)
      }
      val getIntent = InAppSendStatistics.getIntent(inApp.properties.action, "content", inApp, context)
      if (getIntent != null) {
        getIntent.flags = Intent.FLAG_ACTIVITY_NEW_TASK
        context!!.startActivity(getIntent)
      }
    } catch (e: Exception) {
      e.printStackTrace()
    }
  }

  private fun addNewInAppToDismissForever(arguments: Any?) {
    try {
      val obj = JSONObject(arguments as Map<*, *>)
      var inApp = InApp(obj)
      inApp?.let{
        PopUpUtils.addNewInAppToDismissForever(context, it)
      }
    } catch (e: Exception) {
      e.printStackTrace()
    }
  }

  private fun showPopUp(arguments: Any?, result: Result) {
    arguments?.let{
      context?.let{ contxt ->
        val obj = JSONObject(it as Map<*, *>)
//      val view = obj.get(VIEW) as View
        val view = activity?.findViewById<View>(android.R.id.content)?.rootView
        val inAppCode = obj.getString(POP_UP_CODE)
        val imageCloseButtonName = obj.get(CLOSE_BUTTON) as String
        val imageCloseButton = ImageButton(contxt)
        imageCloseButton.setBackgroundColor(Color.TRANSPARENT)
        imageCloseButton.setImageResource(contxt.resources.getIdentifier(imageCloseButtonName, "drawable", contxt.packageName))
        val closeIconDisabled = obj.getBoolean(CLOSE_ICON_DISABLED)

        Indigitall.showPopUp(view, contxt, inAppCode, imageCloseButton, closeIconDisabled, object: ShowInAppCallback(){
          override fun onLoad(p0: String?, p1: WebView?) {
            ContextCompat.getMainExecutor(context).execute {
              result.success(inAppToMap(CALLBACK_SHOW_POPUP, p0))
            }
          }

          override fun onFail(p0: String?, p1: WebView?, p2: String?) {
            ContextCompat.getMainExecutor(context).execute {
              result.error(CALLBACK_ON_ERROR, p0, null)
            }
          }

          override fun onShowTimeFinished(p0: String?, p1: WebView?, p2: Int) {
            val log = Log("IndigitallFlutterPlugin", context)
            log.d("onShowTimeFinished").writeLog()
//            ContextCompat.getMainExecutor(context).execute {
              // result.success(inAppShowTimeFinishedToMap(p0, p2))
//            }
          }

          override fun didClicked() {
            super.didClicked()
            val log = Log("IndigitallFlutterPlugin", context)
            log.d("didClicked").writeLog()
//          ContextCompat.getMainExecutor(context).execute {
//            result.success(popUpActionToMap(CALLBACK_DID_CLICKED))
//          }
          }

          override fun didClosed() {
            super.didClosed()
            val log = Log("IndigitallFlutterPlugin", context)
            log.d("didClosed").writeLog()
//          ContextCompat.getMainExecutor(context).execute {
//            result.success(popUpActionToMap(CALLBACK_DID_CLOSED))
//          }
          }

          override fun didExpired(p0: InApp?, p1: Error?) {
            super.didExpired(p0, p1)
            ContextCompat.getMainExecutor(context).execute {
              result.error(CALLBACK_ON_ERROR, p1?.errorMessage, null)
            }
          }

          override fun didClickOut(p0: InApp?, p1: Error?) {
            super.didClickOut(p0, p1)
            ContextCompat.getMainExecutor(context).execute {
              result.error(CALLBACK_ON_ERROR, p1?.errorMessage, null)
            }
          }

          override fun didShowMoreThanOnce(p0: InApp?, p1: Error?) {
            super.didShowMoreThanOnce(p0, p1)
            ContextCompat.getMainExecutor(context).execute {
              result.error(CALLBACK_ON_ERROR, p1?.errorMessage, null)
            }
          }

          override fun didDismissForever(p0: InApp?, p1: Error?) {
            super.didDismissForever(p0, p1)
            ContextCompat.getMainExecutor(context).execute {
              result.error(CALLBACK_ON_ERROR, p1?.errorMessage, null)
            }
          }

          override fun onSuccess(p0: InApp?) {
            val log = Log("IndigitallFlutterPlugin", context)
            log.d("onSuccess").writeLog()
            super.onSuccess(p0)
          }

          override fun didDismissed() {
            super.didDismissed()
            val log = Log("IndigitallFlutterPlugin", context)
            log.d("didDismissed").writeLog()
//          ContextCompat.getMainExecutor(context).execute {
//            result.success(popUpActionToMap(CALLBACK_DID_DISMISSED))
//          }
          }

        })
      }

    }
  }


  override fun onRequestPermissionsResult(
    requestCode: Int,
    permissions: Array<out String>?,
    grantResults: IntArray?
  ): Boolean {
    if (requestCode == REQUEST_CODE_ASK_PERMISSIONS && grantResults?.isNotEmpty() == true) {
      permissions?.indices?.let{
        for (i in it) {
          if (Manifest.permission.ACCESS_FINE_LOCATION == permissions[i] && grantResults[i] == PackageManager.PERMISSION_GRANTED) {
            // if (PreferenceUtils.getLocationEnabled(cordova.getActivity().getApplicationContext())) {
            ServiceUtils.registerLocationService(context?.applicationContext)
            // }
            return true
          }
        }
      }
    }else{
      return false
    }
    return false
  }

  private fun stringToMap(key: String, value: String): Map<*, *>{
    val map: MutableMap<String, String> = HashMap()
    map[key] = value
    return map
  }

  private fun topicToMap(key: String, value: ArrayList<Map<*, *>>): Map<*, *>{
    val map: MutableMap<String, ArrayList<Map<*, *>>> = HashMap()
    map[key] = value
    return map
  }

  private fun inAppToMap(key: String, inAppcode: String?): Map<*, *>{
    val map: MutableMap<String, Any> = HashMap()
    inAppcode?.let{map[INAPP_CODE] = inAppcode}
//    webView?.let{map[WEBVIEW] = webView}
    val inAppMap: MutableMap<String, Map<*,*>> = HashMap()
    inAppMap[key] = map
    return inAppMap
  }

  private fun popUpActionToMap(action: String): Map<*, *>{
    val map: MutableMap<String, String> = HashMap()
    map[ACTION_POPUP] = action
    return map
  }

  private fun inAppShowTimeFinishedToMap(inAppcode: String?, time: Int): Map<*, *>{
    val map: MutableMap<String, Any> = HashMap()
    map[SHOWTIME] = time
    inAppcode?.let{map[INAPP_CODE] = inAppcode}
//    webView?.let{map[WEBVIEW] = webView}

    val mapShowTime: MutableMap<String, MutableMap<String, Any> > = HashMap()
    mapShowTime[CALLBACK_SHOWTIME_FINISHED] = map
    return mapShowTime
  }


  private fun mapFromDevice(device: Device, key: String): Map<String, Map<*,*>> {
    val MAP_ENABLE = "enable"
    val MAP_DEVICE_ID = "deviceId"
    val MAP_PUSH_TOKEN = "pushToken"
    val MAP_PLATFORM = "platform"
    val MAP_VERSION = "version"
    val MAP_PRODUCT_NAME = "productName"
    val MAP_PRODUCT_VERSION = "productVersion"
    val MAP_OS_NAME = "osName"
    val MAP_OS_VERSION = "osVersion"
    val MAP_DEVICE_BRAND = "deviceBrand"
    val MAP_DEVICE_MODEL = "deviceModel"
    val MAP_OPERATOR = "operator"
    val MAP_DEVICE_TYPE = "deviceType"
    val MAP_APP_VERSION = "appVersion"
    val MAP_LOCALE = "locale"
    val MAP_TIME_ZONE = "timeZone"
    val MAP_TIME_OFFSET = "timeOffset"
    val MAP_EXTERNAL_APPS = "externalApps"
    val MAP_EXTERNAL_CODE = "externalCode"
    val map: MutableMap<String, String> = HashMap()

    device.isEnabled?.let{map[MAP_ENABLE] = it.toString()}
    device.deviceId?.let{map[MAP_DEVICE_ID] = it}
    device.pushToken?.let{map[MAP_PUSH_TOKEN] = it}
    device.platform?.let{map[MAP_PLATFORM] = it}
    device.version?.let{map[MAP_VERSION] = it}
    device.productName?.let{map[MAP_PRODUCT_NAME] = it}
    device.productVersion?.let{map[MAP_PRODUCT_VERSION] = it}
    device.osName?.let{map[MAP_OS_NAME] = it}
    device.osVersion?.let{map[MAP_OS_VERSION] = it}
    device.deviceBrand?.let{map[MAP_DEVICE_BRAND] = it}
    device.deviceModel?.let{map[MAP_DEVICE_MODEL] = it}
    device.operator?.let{map[MAP_OPERATOR] = it}
    device.deviceType?.let{map[MAP_DEVICE_TYPE] = it}
    device.appVersion?.let{map[MAP_APP_VERSION] = it}
    device.locale?.let{map[MAP_LOCALE] = it}
    device.timeZone?.let{map[MAP_TIME_ZONE] = it}
    device.timeOffset?.let{map[MAP_TIME_OFFSET] = it.toString()}
    device.externalCode?.let{map[MAP_EXTERNAL_CODE] = device.externalCode}
    device.externalApps?.let{map[MAP_EXTERNAL_APPS] = jsonArrayFromExternalApps(it).toString()}

    val deviceMap: MutableMap<String, Map<*,*>> = HashMap()
    deviceMap[key] = map
    return deviceMap
  }

  private fun jsonArrayFromExternalApps(externalApps: Array<ExternalApp>?): JSONArray {
    val MAP_ID = "id"
    val MAP_NAME = "name"
    val MAP_CODE = "code"
    val array = JSONArray()
    if (externalApps != null) {
      try {
        for (app in externalApps) {
          val map : MutableMap<String, String> = HashMap()
          app.id?.let{map[MAP_ID] = it.toString()}
          app.name?.let{map[MAP_NAME] = it}
          app.androidCode?.let{map[MAP_CODE] = it}
          array.put(map)
        }
      } catch (ex: JSONException) {
        ex.printStackTrace()
      }
    }
    return array
  }

  private fun parseTopicArray(params: JSONArray?): Array<String?> {
    var topics = arrayOf<String?>()
    try {
      params?.let{
        if (it.length() > 0) {
          topics = arrayOfNulls(it.length())
          for (i in 0 until it.length()) {
            topics[i] = it.getString(i)
          }
        }
      }
    } catch (ex: JSONException) {
      ex.printStackTrace()
    }
    return topics
  }


  private fun mapFromTopic(topic: Topic): Map<*, *> {
    val MAP_CODE = "code"
    val MAP_NAME = "name"
    val MAP_VISIBLE = "visible"
    val MAP_SUBSCRIBED = "subscribed"
    val MAP_PARENT_CODE = "parentCode"
    val map : MutableMap<String, String> = HashMap()
    try {
      topic.code?.let{map[MAP_CODE] = it}
      topic.name?.let{map[MAP_NAME] = it}
      topic.isVisible?.let{map[MAP_VISIBLE] = it.toString()}
      topic.isSubscribed?.let{map[MAP_SUBSCRIBED] = it.toString()}
      topic.parentCode?.let{map[MAP_PARENT_CODE] = it}
    } catch (ex: JSONException) {
      ex.printStackTrace()
    }
    return map
  }

  private fun listFromTopicArray(topics: Array<Topic>): ArrayList<Map<*, *>> {
    val list = arrayListOf<Map<*, *>>()
    for (i in topics.indices){
      list.add(mapFromTopic(topics[i]))
    }
    return list
  }


  private fun mapFromPush(push: Push): Map<*, *> {
    val MAP_ID = "id"
    val MAP_APP_KEY = "appKey"
    val MAP_TITLE = "title"
    val MAP_BODY = "body"
    val MAP_ICON = "icon"
    val MAP_IMAGE = "image"
    val MAP_GIF = "gif"
    val MAP_LAYOUT = "layout"
    val MAP_DATA = "data"
    val MAP_SILENT = "silent"
    val MAP_SECURE_DATA = "securedData"
    val MAP_ACTION = "action"
    val MAP_BUTTONS = "buttons"
    val MAP_SENDING_ID = "sendingId"
    val MAP_CAMPAIGN_ID = "CampaignId"
    val MAP_PUSH_ID = "pushId"
    val json = JSONObject()
    try {
      push.id?.let{
        json.put(MAP_ID , push.id.toString())
        json.put(MAP_PUSH_ID, push.id.toString())
      }
      push.appKey?.let{json.put(MAP_APP_KEY, it)}
      push.title?.let{json.put(MAP_TITLE, it)}
      push.body?.let{json.put(MAP_BODY, it)}
      push.icon?.let{json.put(MAP_ICON, it)}
      push.image?.let{json.put(MAP_IMAGE, it)}
      push.gif?.let{json.put(MAP_GIF, it.toString())}
      push.layout?.let{json.put(MAP_LAYOUT,it.toString())}
      push.data?.let{json.put(MAP_DATA, it)}
      push.isSilent?.let{json.put(MAP_SILENT, it.toString())}
      push.action?.let{json.put(MAP_ACTION, mapFromAction(it).toString())}
      push.securedData?.let{json.put(MAP_SECURE_DATA, it)}
      push.sendingId?.let{json.put(MAP_SENDING_ID, it)}
      push.campaignId?.let{json.put(MAP_CAMPAIGN_ID, it)}

      if (push.buttons != null) {
        for (i in push.buttons.indices) {
          val pushButton = push.buttons[i]
          json.put(MAP_BUTTONS, mapFromPushButtons(pushButton).toString())
        }
      }
    } catch (ex: JSONException) {
      ex.printStackTrace()
    }

    val pushMap: MutableMap<String, String> = HashMap()
    pushMap[CALLBACK_GET_PUSH] = push.toString()
    return pushMap
  }

  private fun mapFromAction(pushAction: BaseAction): JSONObject {
    val MAP_DESTROY = "destroy"
    val MAP_TOPICS = "topics"
    val MAP_TYPE = "type"
    val MAP_TYPE_APP = "app"
    val MAP_TYPE_URL = "url"
    val MAP_TYPE_CALL = "call"
    val MAP_TYPE_MARKET = "market"
    val MAP_TYPE_SHARE = "share"
    val map = JSONObject()
    try {
      pushAction.isDestroy?.let{map.put(MAP_DESTROY, it.toString())}
      pushAction.topics?.let{map.put(MAP_TOPICS,it.toString())}
      pushAction.type?.let{map.put(MAP_TYPE, it.toString())}
      pushAction.app?.let{map.put(MAP_TYPE_APP, it)}
      pushAction.url?.let{map.put(MAP_TYPE_URL, it)}
      pushAction.call?.let{map.put(MAP_TYPE_CALL, it)}
      pushAction.market?.let{map.put(MAP_TYPE_MARKET, it)}
      pushAction.share?.let{map.put(MAP_TYPE_SHARE, it)}
    } catch (ex: JSONException) {
      ex.printStackTrace()
    }
    return map
  }

  private fun mapFromPushButtons(pushButton: PushButton): Map<*, *> {
    val MAP_LABEL = "label"
    val MAP_ACTION = "action"
    val MAP_TOPICS = "topics"
    val map : MutableMap<String, String> = HashMap()
    try {
      pushButton.label?.let{map[MAP_LABEL] = it}
      pushButton.action?.let{map[MAP_ACTION] = mapFromAction(it).toString()}
      pushButton.topics?.let{map[MAP_TOPICS] = it.toString()}
    } catch (ex: JSONException) {
      ex.printStackTrace()
    }
    return map
  }

  private fun inAppToMap(inApp: InApp):  Map<*, *>{
    val INAPPID = "inAppId"
    val LAST_VERSION_ID = "lastVersionId"
    val SHOW_ONCE = "showOnce"
    val RENEWAL_TIME = "renewalTime"
    val CREATION_DATE = "creationDate"
    val EXPIRED_DATE = "expiredDate"
    val PROPERTIES = "properties"
    val SCHEMA = "schema"

    val map: MutableMap<String, Any> = HashMap()
    inApp.inAppId?.let{map[INAPPID] = it}
    inApp.lastVersionId?.let{map[LAST_VERSION_ID] = it}
    inApp.isShowOnce?.let{map[SHOW_ONCE] = it}
    inApp.renewalTime?.let{map[RENEWAL_TIME] = it}
    //inApp.numberOfClicks?.let{map[NUMBER_OF_CLICKS] = it}
    inApp.creationDate?.let{map[CREATION_DATE] = it}
    inApp.expiredDate?.let{map[EXPIRED_DATE] = it}
    inApp.properties?.let{
      map[PROPERTIES] = inAppPropertiesToMap(it)
    }
    inApp.schema?.let{
      map[SCHEMA] = inAppSchemaToMap(it)
    }
    val inAppMap: MutableMap<String, Map<*,*>> = HashMap()
    inAppMap[CALLBACK_GET_INAPP] = map
    return inAppMap
  }

  private fun inAppPropertiesToMap(inAppProperties: InAppProperties):  Map<*, *>{
    val ACTION = "action"
    val CONTENT_URL = "contentUrl"
    val NUMBER_OF_SHOWS = "numberOfShows"
    val NUMBER_OF_CLICKS = "numberOfClicks"
    val SHOWTIME = "showTime"
    val DISMISS_FOREVER = "dismissForever"
    val LAYOUT = "layout"
    val map: MutableMap<String, Any> = HashMap()
    inAppProperties.action?.let{
      map[ACTION] = inAppActionToMap(it).toString()
    }
    inAppProperties.contentUrl?.let{map[CONTENT_URL] = it}
    inAppProperties.showTime?.let{map[SHOWTIME] = it}
    inAppProperties.numberOfShows?.let{map[NUMBER_OF_SHOWS] = it}
    inAppProperties.numberOfClicks?.let{map[NUMBER_OF_CLICKS] = it}
    inAppProperties.dismissForever?.let{map[DISMISS_FOREVER] = it}
//    inAppProperties.layout?.let{
//      map[LAYOUT] = inAppLayoutToMap(it)
//    }
    return map
  }

  private fun inAppSchemaToMap(inAppSchema: InAppSchema):  Map<*, *> {
    val CODE = "code"
    val WIDTH = "width"
    val HEIGHT = "height"
    val map: MutableMap<String, Any> = HashMap()
    inAppSchema.width?.let{map[WIDTH] = it}
    inAppSchema.code?.let{map[CODE] = it}
    inAppSchema.height?.let{map[HEIGHT] = it}
    return map
  }

  private fun inAppActionToMap(inAppAction: BaseAction):  JSONObject?{
    return (mapFromAction(inAppAction))
  }

//  private fun inAppLayoutToMap(inAppLayout: InAppLayout): Map<*, *> {
//
//  }

}
