package com.indigitall.indigitall_flutter_plugin

import android.content.Context
import androidx.core.content.ContextCompat

import io.flutter.plugin.common.MethodChannel.Result
import org.json.JSONException
import org.json.JSONObject


import com.indigitall.android.commons.models.Push
import com.indigitall.android.commons.models.PushAction
import com.indigitall.android.commons.models.PushButton
import com.indigitall.android.commons.models.Error
import org.json.JSONArray

import com.indigitall.android.inbox.models.InboxNotification
import com.indigitall.android.inbox.Inbox
import com.indigitall.android.inbox.Utils.InboxAuthListener
import com.indigitall.android.inbox.models.InboxCounters
import com.indigitall.android.inbox.callbacks.InboxNotificationsCallback
import com.indigitall.android.inbox.models.InboxStatus
import com.indigitall.android.inbox.models.InboxCategory
import com.indigitall.android.inbox.callbacks.InboxBaseCallback
import com.indigitall.android.inbox.callbacks.InboxNextPageCallback
import com.indigitall.android.inbox.callbacks.InboxCountersCallback
import com.indigitall.android.inbox.callbacks.InboxCallback


/** IndigitallInboxFlutterPlugin */
class IndigitallInboxFlutterPlugin(val context: Context?) : InboxAuthListener {

  private val AUTH_VAR = "auth"
  private val CALLBACK_GET_INBOX = "callbackGetInbox"
  private val CALLBACK_GET_MESSAGE_COUNT = "callbackGetMessageCount"
  private val CALLBACK_GET_NEXT_PAGE = "callbackGetNextPage"
  private val CALLBACK_MASSIVE_EDIT = "callbackMassiveEdit"
  private val CALLBACK_GET_INFO_NOTIFICATION = "callbackGetInfoNotification"
  private val CALLBACK_MODIFY_STATUS = "callbackModifyStatus"

  private val ACTION_ERROR = "error"

  var jsonAuth: JSONObject ?= null


  override fun getAuthConfig(): JSONObject {
    var json = JSONObject()
    jsonAuth?.let{
      json = it
    }
    return json
  }

  fun getInbox(arguments: Any?, inboxCallback: InboxCallback ,result: Result){
    context?.let {
      arguments?.let { args ->
        jsonAuth = JSONObject(args as Map<*, *>)
      }
      Inbox.Companion.getInbox(it, object : InboxCallback(it) {
        override fun onSuccess(inbox: Inbox) {
          ContextCompat.getMainExecutor(context).execute {
            inboxCallback.onSuccess(inbox)
            val inboxMap: MutableMap<String, Map<*,*>> = HashMap()
            inboxMap[CALLBACK_GET_INBOX] = mapFromInbox(inbox, null)
            result.success(inboxMap)
          }
        }

        override fun onError(error: Error) {
          ContextCompat.getMainExecutor(context).execute {
            result.error(error.errorCode?.name, error.errorMessage, null)
          }
        }
      })
    }
  }

  fun getMessagesCount(arguments: Any?, result: Result){
    context?.let {
      arguments?.let { args ->
        jsonAuth = JSONObject(args as Map<*, *>)
      }
      Inbox.Companion.getMessagesCount(it, object : InboxCountersCallback(it) {
        override fun onSuccess(inboxCounters: InboxCounters?) {
          ContextCompat.getMainExecutor(context).execute {
            result.success(mapFromMessagesCount(inboxCounters))
          }

        }

        override fun onError(error: Error) {
          ContextCompat.getMainExecutor(context).execute {
            result.error(error.errorCode?.name, error.errorMessage, null)
          }
        }
      })
    }
  }

  fun getNextPage(myInbox: Inbox?, arguments: Any?, result: Result){
    context?.let {
      arguments?.let { args ->
        jsonAuth = JSONObject(args as Map<*, *>)
      }
      myInbox?.getNextPage(it, object : InboxNextPageCallback(it) {
        override fun onSuccess(inbox: Inbox, arrayList: ArrayList<InboxNotification>?) {
          ContextCompat.getMainExecutor(context).execute {
            val inboxMap: MutableMap<String, Map<*,*>> = HashMap()
            inboxMap[CALLBACK_GET_NEXT_PAGE] = mapFromInbox(inbox, arrayList)
            result.success(inboxMap)
          }
        }

        override fun onFail(error: Error) {
          ContextCompat.getMainExecutor(context).execute {
            result.error(error.errorCode?.name, error.errorMessage, null)
          }
        }
      })
    }
  }

  fun massiveEdit(myInbox: Inbox?, arguments: Any?, result: Result){
    val MAP_SENDING_ID_LIST = "sendingIdList"
    val MAP_STATUS = "status"
    var status: String? = null
    var jsonArraySendingIdList: JSONArray? = null
    val sendingIdList = ArrayList<Int>()
    arguments?.let{args ->
      val params = JSONObject(args as Map<*, *>)
      try {
        if (params.has(AUTH_VAR)) jsonAuth = params.getJSONObject(AUTH_VAR)
        if (params.has(MAP_SENDING_ID_LIST)) jsonArraySendingIdList =
            params.getJSONArray(MAP_SENDING_ID_LIST)
        if (params.has(MAP_STATUS)) status = params.getString(MAP_STATUS)

        if (jsonArraySendingIdList != null) {
          for (i in 0 until jsonArraySendingIdList!!.length()) {
            sendingIdList.add(jsonArraySendingIdList!!.getInt(i))
          }
        }
      } catch (ex: JSONException) {
        ex.printStackTrace()
      }
      var inboxStatus: InboxStatus? = null
      if (status != null) {
        inboxStatus = if (status?.toLowerCase() == "click") {
          InboxStatus.CLICK
        } else if (status?.toLowerCase() == "delete" || status?.toLowerCase() == "deleted") {
          InboxStatus.DELETED
        } else {
          InboxStatus.SENT
        }
        context?.let {
          myInbox?.massiveEditNotifications(
            it,
            sendingIdList,
            inboxStatus,
            object : InboxBaseCallback(it) {
              override fun onSuccess() {
                ContextCompat.getMainExecutor(context).execute {
                  result.success(stringToMap(CALLBACK_MASSIVE_EDIT, "success"))
                }
              }

              override fun onFail(error: Error) {
                ContextCompat.getMainExecutor(context).execute {
                  result.error(error.errorCode?.name, error.errorMessage, null)
                }
              }
            })
        }
      } else {
        ContextCompat.getMainExecutor(context).execute {
          result.error("massiveEdit", "No status params", null)
        }
      }
    }
  }

  fun getInfoFromNotification(
    myInbox: Inbox?,
    arguments: Any?,
    result: Result
  ){
    val MAP_SENDING_ID = "sendingId"
    var sendingId = 0
    arguments?.let{args ->
      val params = JSONObject(args as Map<*, *>)
      try {
        if (params.has(AUTH_VAR)) jsonAuth = params.getJSONObject(AUTH_VAR)
        if (params.has(MAP_SENDING_ID)) sendingId = params.getInt(MAP_SENDING_ID)
      } catch (ex: JSONException) {
        ex.printStackTrace()
      }
      context?.let {
        myInbox?.getInfoFromNotification(
          it,
          sendingId,
          object : InboxNotificationsCallback(it) {
            override fun onSuccess(inboxNotification: InboxNotification?) {
              if (inboxNotification != null) {
                ContextCompat.getMainExecutor(context).execute {
                  val inboxNotificationMap: MutableMap<String, Map<*,*>> = HashMap()
                  inboxNotificationMap[CALLBACK_GET_INFO_NOTIFICATION] = mapFromInboxNotification(inboxNotification)
                  result.success(inboxNotificationMap)
                }
              } else {
                ContextCompat.getMainExecutor(context).execute {
                  result.error("getInfoFromNotification", "inboxNotification is null", null)
                }
              }
            }

            override fun onError(error: Error) {
              ContextCompat.getMainExecutor(context).execute {
                result.error(error.errorCode?.name, error.errorMessage, null)
              }
            }
          })
      }
    }
  }

  fun modifyStatus(
    myInbox: Inbox?,
    arguments: Any?,
    result: Result
  ){
    val MAP_SENDING_ID = "sendingId"
    val MAP_STATUS = "status"
    var status: String? = null
    var sendingId = 0
    arguments?.let{args ->
      val params = JSONObject(args as Map<*, *>)
      try {
        if (params.has(MAP_SENDING_ID)) sendingId = params.getInt(MAP_SENDING_ID)
        if (params.has(MAP_STATUS)) status = params.getString(MAP_STATUS)
        if (params.has(AUTH_VAR)) jsonAuth = params.getJSONObject(AUTH_VAR)
      } catch (ex: JSONException) {
        ex.printStackTrace()
      }
      var inboxStatus: InboxStatus? = null
      if (status != null) {
        inboxStatus = if (status?.toLowerCase() == "click") {
          InboxStatus.CLICK
        } else if (status?.toLowerCase() == "delete" || status?.toLowerCase() == "deleted") {
          InboxStatus.DELETED
        } else {
          InboxStatus.SENT
        }
        context?.let {
          myInbox?.modifyStatusFromNotification(
            it,
            sendingId,
            inboxStatus,
            object : InboxNotificationsCallback(it) {
              override fun onSuccess(inboxNotification: InboxNotification?) {
                if (inboxNotification != null) {
                  ContextCompat.getMainExecutor(context).execute {
                    val inboxNotificationMap: MutableMap<String, Map<*,*>> = HashMap()
                    inboxNotificationMap[CALLBACK_MODIFY_STATUS] = mapFromInboxNotification(inboxNotification)
                    result.success(inboxNotificationMap)
                  }
                } else {
                  ContextCompat.getMainExecutor(context).execute {
                    result.error("getInfoFromNotification", "inboxNotification is null", null)
                  }
                }
              }

              override fun onError(error: Error) {
                ContextCompat.getMainExecutor(context).execute {
                  result.error(error.errorCode?.name, error.errorMessage, null)
                }
              }
            })
        }
      }
    }
  }

  private fun mapFromMessagesCount(inboxCounters: InboxCounters?): Map<String, Map<*,*>> {
    val MAP_CLICK = "click"
    val MAP_SENT = "sent"
    val MAP_DELETED = "deleted"
    val MAP_LAST_ACCESS = "lastAccess"
    val MAP_COUNT = "count"
    val map: MutableMap<String, String> = HashMap()
    try {
      inboxCounters?.click?.let { map[MAP_CLICK] = it.toString() }
      inboxCounters?.sent?.let { map[MAP_SENT] = it.toString() }
      inboxCounters?.deleted?.let { map[MAP_DELETED] = it.toString() }
      inboxCounters?.getUnread()?.let {
        it.lastAccess?.let { it1 -> map[MAP_LAST_ACCESS] = it1.toString() }
        it.getCount()?.let { it1 -> map[MAP_COUNT] = it1.toString() }
      }
    } catch (ex: Exception) {
      ex.printStackTrace()
    }
    val messageCountMap: MutableMap<String, Map<*,*>> = HashMap()
    messageCountMap[CALLBACK_GET_MESSAGE_COUNT] = map
    return messageCountMap
  }

  private fun mapFromInbox(
    inbox: Inbox,
    newNotifications: ArrayList<InboxNotification>?
  ): Map<String, String>  {
    val MAP_LAST_ACCESS = "lastAccess"
    val MAP_COUNT = "count"
    val MAP_PAGESIZE = "pageSize"
    val MAP_PAGE = "numPage"
    val MAP_NOTIFICATIONS = "notifications"
    val MAP_NEW_NOTIFICAIONS = "newNotifications"
    val map: MutableMap<String, String> = HashMap()
    try {
      inbox.lastAccess?.let{map[MAP_LAST_ACCESS] = it.toString()}
      inbox.count?.let{map[MAP_COUNT] =  it.toString()}
      inbox.pageSize?.let{map[MAP_PAGESIZE] =  it.toString()}
      inbox.page?.let{map[MAP_PAGE] =  it.toString()}
      inbox.notifications?.let{map[MAP_NOTIFICATIONS] = mapArrayFromInboxNotifications(it).toString()}
      newNotifications?.let { map[MAP_NEW_NOTIFICAIONS] = mapArrayFromInboxNotifications(it).toString()}
    } catch (ex: Exception) {
      ex.printStackTrace()
    }
    return map
  }

  private fun mapArrayFromInboxNotifications(inboxNotifications: ArrayList<InboxNotification>?): JSONArray {
    val MAP_ID = "id"
    val MAP_SENTAT = "sentAt"
    val MAP_STATUS = "status"
    val MAP_SENDINGID = "sendingId"
    val MAP_CAMPAIGNID = "campaignId"
    val MAP_MESSAGE = "message"
    val MAP_CATEGORY = "category"
    val array = JSONArray()
    if (inboxNotifications != null) {
      try {
        for (inboxNotification in inboxNotifications) {
          val json = JSONObject()
          inboxNotification.id?.let{json.put(MAP_ID, it)}
          inboxNotification.sentAt?.let{json.put(MAP_SENTAT, it.toString())}
          inboxNotification.status?.let{json.put(MAP_STATUS,  it.toString())}
          inboxNotification.sendingId?.let{json.put(MAP_SENDINGID, it.toString())}
          inboxNotification.campaignId?.let{json.put(MAP_CAMPAIGNID,  it.toString())}
          inboxNotification.message?.let{ json.put(MAP_MESSAGE, mapFromPush(it).toString()) }
          inboxNotification.category?.let{ json.put(MAP_CATEGORY, mapFromCategory(it).toString()) }
          array.put(json.toString())
        }
      } catch (ex: JSONException) {
        ex.printStackTrace()
      }
    }
    return array
  }

  private fun errorToMap(error: Error): Map<*, *>{
    val map: MutableMap<String, MutableMap<String, String>> = HashMap()
    val mapError :MutableMap<String, String> = HashMap()
    error.errorCode?.name?.let {
      error.errorMessage?.let{it1 -> mapError[it] = it1}
    }
    map[ACTION_ERROR] = mapError
    return map
  }

  private fun mapFromInboxNotification(inboxNotification: InboxNotification): Map<String, String>  {
    val MAP_ID = "id"
    val MAP_SENTAT = "sentAt"
    val MAP_STATUS = "status"
    val MAP_SENDINGID = "sendingId"
    val MAP_CAMPAIGNID = "campaignId"
    val MAP_MESSAGE = "message"
    val MAP_CATEGORY = "category"
    val map: MutableMap<String, String> = HashMap()
    try {
      inboxNotification.id?.let{map[MAP_ID] =  it}
      inboxNotification.sentAt?.let{map[MAP_SENTAT] =  it.toString()}
      inboxNotification.status?.let{map[MAP_STATUS] =  it.toString()}
      inboxNotification.sendingId?.let{map[MAP_SENDINGID] =  it.toString()}
      inboxNotification.campaignId?.let{map[MAP_CAMPAIGNID] =  it.toString()}
      inboxNotification.message?.let{ map[MAP_MESSAGE] = mapFromPush(it).toString() }
      inboxNotification.category?.let{ map[MAP_CATEGORY] = mapFromCategory(it).toString() }
    } catch (ex: Exception) {
      ex.printStackTrace()
    }
    return map
  }


  private fun mapFromCategory(category: InboxCategory): Map<String, String>  {
    val MAP_ID = "id"
    val MAP_NAME = "name"
    val MAP_CODE = "code"
    val map: MutableMap<String, String> = HashMap()
    try {
      category.id?.let{map[MAP_ID] =  it.toString()}
      category.name?.let{map[MAP_NAME] =  it.toString()}
      category.code?.let{map[MAP_CODE] =  it.toString()}
    } catch (ex: Exception) {
      ex.printStackTrace()
    }
    return map
  }

  private fun mapFromPush(push: Push): JSONObject {
    val MAP_ID = "id"
    val MAP_APP_KEY = "appKey"
    val MAP_TITLE = "title"
    val MAP_BODY = "body"
    val MAP_ICON = "icon"
    val MAP_IMAGE = "image"
    val MAP_GIF = "gif"
    val MAP_LAYOUT = "layout"
    val MAP_DATA = "data"
    val MAP_SILENT = "silent"
    val MAP_SECURE_DATA = "securedData"
    val MAP_ACTION = "action"
    val MAP_BUTTONS = "buttons"
    val MAP_SENDING_ID = "sendingId"
    val MAP_CAMPAIGN_ID = "CampaignId"
    val MAP_PUSH_ID = "pushId"
    val json = JSONObject()
    try {
      push.id?.let{
        json.put(MAP_ID , push.id.toString())
        json.put(MAP_PUSH_ID, push.id.toString())
      }
      push.appKey?.let{json.put(MAP_APP_KEY, it)}
      push.title?.let{json.put(MAP_TITLE, it)}
      push.body?.let{json.put(MAP_BODY, it)}
      push.icon?.let{json.put(MAP_ICON, it)}
      push.image?.let{json.put(MAP_IMAGE, it)}
      push.gif?.let{json.put(MAP_GIF, it.toString())}
      push.layout?.let{json.put(MAP_LAYOUT,it.toString())}
      push.data?.let{json.put(MAP_DATA, it)}
      push.isSilent?.let{json.put(MAP_SILENT, it.toString())}
      push.action?.let{json.put(MAP_ACTION, mapFromAction(it).toString())}
      push.securedData?.let{json.put(MAP_SECURE_DATA, it)}
      push.sendingId?.let{json.put(MAP_SENDING_ID, it)}
      push.campaignId?.let{json.put(MAP_CAMPAIGN_ID, it)}

      if (push.buttons != null) {
        for (i in push.buttons.indices) {
          val pushButton = push.buttons[i]
          json.put(MAP_BUTTONS, mapFromPushButtons(pushButton).toString())
        }
      }
    } catch (ex: JSONException) {
      ex.printStackTrace()
    }
    return json
  }

  private fun mapFromAction(pushAction: PushAction): JSONObject {
    val MAP_DESTROY = "destroy"
    val MAP_TOPICS = "topics"
    val MAP_TYPE = "type"
    val MAP_TYPE_APP = "app"
    val MAP_TYPE_URL = "url"
    val MAP_TYPE_CALL = "call"
    val MAP_TYPE_MARKET = "market"
    val MAP_TYPE_SHARE = "share"
    val json = JSONObject()
    try {
      pushAction.isDestroy?.let{json.put(MAP_DESTROY, it.toString())}
      pushAction.topics?.let{json.put(MAP_TOPICS,it.toString())}
      pushAction.type?.let{json.put(MAP_TYPE, it.toString())}
      pushAction.app?.let{json.put(MAP_TYPE_APP, it)}
      pushAction.url?.let{json.put(MAP_TYPE_URL, it)}
      pushAction.call?.let{json.put(MAP_TYPE_CALL, it)}
      pushAction.market?.let{json.put(MAP_TYPE_MARKET, it)}
      pushAction.share?.let{json.put(MAP_TYPE_SHARE, it)}
    } catch (ex: JSONException) {
      ex.printStackTrace()
    }
    return json
  }

  private fun mapFromPushButtons(pushButton: PushButton): Map<*, *> {
    val MAP_LABEL = "label"
    val MAP_ACTION = "action"
    val MAP_TOPICS = "topics"
    val map : MutableMap<String, String> = HashMap()
    try {
      pushButton.label?.let{map[MAP_LABEL] = it}
      pushButton.action?.let{map[MAP_ACTION] = mapFromAction(it).toString()}
      pushButton.topics?.let{map[MAP_TOPICS] = it.toString()}
    } catch (ex: JSONException) {
      ex.printStackTrace()
    }
    return map
  }

  private fun stringToMap(key: String, value: String): Map<*, *>{
    val map: MutableMap<String, String> = HashMap()
    map[key] = value
    return map
  }
}
