
#import "IndigitallInboxFlutterPlugin.h"

@implementation IndigitallInboxFlutterPlugin 

NSString * CALLBACK_GET_INBOX = @"callbackGetInbox";
NSString * CALLBACK_GET_MESSAGE_COUNT = @"callbackGetMessageCount";
NSString * CALLBACK_GET_NEXT_PAGE = @"callbackGetNextPage";
NSString * CALLBACK_MASSIVE_EDIT = @"callbackMassiveEdit";
NSString * CALLBACK_GET_INFO_NOTIFICATION = @"callbackGetInfoNotification";
NSString * CALLBACK_MODIFY_STATUS = @"callbackModifyStatus";

@synthesize auth;

//+ (void) registerWithRegistrar:(NSObject<FlutterPluginRegistrar> *)registrar {
//    FlutterMethodChannel *channel = [FlutterMethodChannel methodChannelWithName:@"indigitall_inbox_flutter_plugin" binaryMessenger:registrar.messenger];
//    [channel setMethodCallHandler:^(FlutterMethodCall * _Nonnull call, FlutterResult  _Nonnull result) {
//
//    }];
//}

- (void) setAuth: (NSDictionary *)auth {
    self.auth = auth;
}

- (NSDictionary *)getAuthConfig {
    NSDictionary *dic = [NSDictionary new];
    if (auth != nil) dic = auth;
    return dic;
}

- (void) getInboxMessagesCountWithArguments:(NSDictionary *_Nullable)params result: (FlutterResult _Nonnull)result{
    if (params != nil) auth = params;
    [Inbox getMessagesCountWithSuccess:^(INInboxCounters * _Nonnull counters) {
        [self successWithInboxCounter:counters result: result];
    } onError:^(INError * _Nonnull error) {
        [self jsonFromError:error withResult:result];
    }];
}

- (void) getInboxWithArguments:(NSDictionary *_Nullable)params success: (void(^)(Inbox *inbox))onSuccess result:(FlutterResult _Nonnull)result {
    if (params != nil) auth = params;
    [Inbox getInboxWithSuccess:^(Inbox * _Nonnull inbox) {
        onSuccess(inbox);
        [self successWithInbox:inbox newNotifications:nil methodGetInbox:true result: result];
    } onError:^(INError * _Nonnull error) {
        [self jsonFromError:error withResult:result];
    }];
       
 
}

- (void) getNextPageWithArguments:(NSDictionary *_Nullable)params inbox: (Inbox *)inbox result:(FlutterResult _Nonnull)result {
    if (params != nil) auth = params;
    if (inbox != nil){
        [inbox getNextPageWithSuccess:^(Inbox * _Nonnull inbox, NSArray<INInboxNotification *> * _Nonnull newNotifications){
            [self successWithInbox:inbox newNotifications:newNotifications methodGetInbox:false result: result];
        } onError:^(INError * _Nonnull error) {
            [self jsonFromError:error withResult:result];
        }];
    }else{
        [self jsonFromError: [[INError alloc]initWithErrorCode:3020 errorMessage:@"getNextPage: inbox object is null"] withResult:result];
    }
}

- (void) massiveEditNotificationsWithSendingIdsList: (NSDictionary *)params inbox:(Inbox *)inbox result: (FlutterResult _Nonnull)result {
    NSArray<NSNumber *>* sendingIdList = params[@"sendingIdList"];
    NSString *status = params[@"status"];
    NSString *statusInbox;
    if (params[@"auth"] != nil) auth = params[@"auth"];
    if ([status isEqualToString:@"click"]){
        statusInbox = InboxStatus.CLICK;
    }else if ([status isEqualToString:@"deleted"]){
        statusInbox = InboxStatus.DELETED;
    }else{
        statusInbox = InboxStatus.SENT;
    }
    if (inbox != nil){
        [inbox massiveEditNotificationsWithSendingIdsList:sendingIdList status:statusInbox onSuccess:^{
            [self resultFromString:@"success" withAction:CALLBACK_MASSIVE_EDIT withResult:result];
        } onError:^(INError * _Nonnull error) {
            [self jsonFromError:error withResult:result];
        }];
    }else{
        [self jsonFromError: [[INError alloc]initWithErrorCode:3020 errorMessage:@"massiveEditNotificationsWithSendingIdsList: inbox object is null"] withResult:result];
    }
    
}

- (void) getInfoFromNotificationWithSendingId: (NSDictionary *)params inbox:(Inbox *)inbox result: (FlutterResult _Nonnull)result {
    if (inbox != nil){
        if (params[@"auth"] != nil) auth = params[@"auth"];
        NSNumber *sendingId = params[@"sendingId"];
        [inbox getInfoFromNotificationWithSendingId:sendingId onSuccess:^(INInboxNotification * _Nonnull inboxNotification) {
            [self successWithInboxNotification:inboxNotification callback:CALLBACK_GET_INFO_NOTIFICATION result: result];
        } onError:^(INError * _Nonnull error) {
            [self jsonFromError:error withResult:result];
        }];
    }else{
        [self jsonFromError: [[INError alloc]initWithErrorCode:3020 errorMessage:@"getInfoFromNotificationWithSendingId: inbox object is null"] withResult:result];
    }

}

- (void) modifyStatusFromNotificationWithSendingId: (NSDictionary *)params inbox:(Inbox *)inbox result: (FlutterResult _Nonnull)result {
    NSNumber * sendingId = params[@"sendingId"];
    NSString *status = params[@"status"];
    if (params[@"auth"] != nil) auth = params[@"auth"];
    if (inbox != nil){
        [inbox modifyStatusFromNotificationWithSendingId:sendingId status:status onSuccess:^(INInboxNotification * _Nonnull inboxNotification) {
            [self successWithInboxNotification:inboxNotification callback:CALLBACK_MODIFY_STATUS result: result];
        } onError:^(INError * _Nonnull error) {
            [self jsonFromError:error withResult:result];
        }];
    }else{
        [self jsonFromError: [[INError alloc]initWithErrorCode:3020 errorMessage:@"modifyStatusFromNotificationWithSendingId: inbox object is null"] withResult:result];
    }
    
}

- (NSDictionary *) jsonFromMessagesCounter: (INInboxCounters *) inboxCounter{
    NSMutableDictionary *obj = [[NSMutableDictionary alloc]init];
    if (inboxCounter != nil) {
        obj = [@{
            @"click" : @(inboxCounter.click) ?: 0,
            @"sent": @(inboxCounter.sent) ?: 0,
            @"deleted" : @(inboxCounter.deleted) ?: 0,
            @"lastAccess": inboxCounter.unread.lastAccess ?: @"",
            @"count": @(inboxCounter.unread.count) ?: 0
        } mutableCopy];
    }
    return [@{
        CALLBACK_GET_MESSAGE_COUNT : [obj copy],
    } copy];
}

- (NSDictionary *) jsonFromInbox: (Inbox *) inbox newNotifications:(NSArray<INInboxNotification *> *)newNotifications callback: (NSString *)callback {
    NSMutableDictionary *obj = [[NSMutableDictionary alloc]init];
    if (inbox != nil) {
        obj = [@{
            @"lastAccess" : inbox.lastAccess ?: @"",
            @"count": @(inbox.count) ?: 0,
            @"pageSize" : inbox.pageSize ?: 0,
            @"numPage": inbox.page ?: 0,
            @"notifications": [self jsonArrayFromInboxNotification: inbox.notifications] ?:@"",
            @"newNotifications": [self jsonArrayFromInboxNotification: newNotifications] ?:@""
        } mutableCopy];
    }
    
    return [@{
        callback : [obj copy],
    } copy];
}

- (NSArray<NSString *>*) jsonArrayFromInboxNotification: (NSArray<INInboxNotification *>* _Nullable) inboxNotifications{
    NSMutableArray<NSString *>* array = [NSMutableArray new];
    @try{
        if (inboxNotifications && inboxNotifications.count > 0){
            for (int i=0;i<inboxNotifications.count;i++){
                NSMutableDictionary *obj = [[NSMutableDictionary alloc]init];
                if (inboxNotifications[i].idInboxNotification) [obj setValue:inboxNotifications[i].idInboxNotification forKey:@"id"];
                if (inboxNotifications[i].sentAt) [obj setValue:inboxNotifications[i].sentAt forKey:@"sentAt"];
                if (inboxNotifications[i].inboxStatus) [obj setValue:inboxNotifications[i].inboxStatus forKey:@"status"];
                if (inboxNotifications[i].sendingId) [obj setValue:inboxNotifications[i].sendingId forKey:@"sendingId"];
                if (inboxNotifications[i].campaignId) [obj setValue:inboxNotifications[i].campaignId forKey:@"campaignId"];
                if (inboxNotifications[i].message) [obj setValue:[self jsonFromPush:inboxNotifications[i].message] forKey:@"message"];
                if (inboxNotifications[i].category) [obj setValue:[self jsonFromCategory:inboxNotifications[i].category] forKey:@"category"];
             
                NSError * err;
                NSData * jsonData = [NSJSONSerialization dataWithJSONObject:obj options:0 error:&err];
                NSString * myString = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
                if (myString != nil) [array addObject:myString];
                
            }
        }
    } @catch (NSException *exception) {
        NSLog(@"Error jsonArrayFromInboxNotification %@", exception);
    }
    if (array.count > 0){
        return [array copy];
    }else{
        return nil;
    }
}


- (NSDictionary *) jsonFromCategory: (InboxCategory *) category{
    NSMutableDictionary *obj = [[NSMutableDictionary alloc]init];
    if (category != nil) {
        obj = [@{
            @"id" : @(category.idCategory) ?: 0,
            @"name": category.name ?: @"",
            @"code": category.code ?: @""
        } mutableCopy];
    }
    return [obj mutableCopy];
}

- (NSDictionary *) jsonFromPush: (INPush *) message{
    NSMutableDictionary *obj = [[NSMutableDictionary alloc]init];
    if (message != nil) {
        if (message.title) [obj setValue:message.title forKey:@"title" ];
        if (message.body) [obj setValue:message.body forKey:@"body" ];
        if (message.image) [obj setValue:message.image forKey:@"image" ];
        if (message.gif) [obj setValue:message.gif forKey:@"gif" ];
        if (message.action) [obj setValue:[self jsonFromPushAction:message.action] forKey:@"action" ];
        if (message.buttons) [obj setValue: [self jsonArrayFromPushButtons:message.buttons] forKey:@"buttons" ];
        if (message.data) [obj setValue:message.data forKey:@"data" ];
        if (message.securedData) [obj setValue:message.securedData forKey:@"securedData" ];
    }
    return [obj copy];
}


- (NSDictionary *) jsonFromPushAction: (INPushAction *) action{
    NSMutableDictionary *obj = [[NSMutableDictionary alloc]init];
    if (action != nil) {
        if (action.url) [obj setValue:action.url forKey:@"url"];
        if (action.app) [obj setValue:action.app forKey:@"app"];
        if (action.market) [obj setValue:action.market forKey:@"market"];
        if (action.share) [obj setValue:action.share forKey:@"share"];
        if (action.call) [obj setValue:action.call forKey:@"call"];
        if (action.noAction) [obj setValue:action.noAction forKey:@"noAction"];
        if (action.wallet) [obj setValue:action.wallet forKey:@"wallet"];
        if (action.destroy) [obj setValue:@(action.destroy) forKey:@"destroy"];
    }
    
    return [obj copy];
}

- (NSArray<NSDictionary *>*) jsonArrayFromPushButtons: (NSArray<INPushButton *>*) buttons{
    NSMutableArray<NSDictionary *>* array = [NSMutableArray new];
    @try{
        if (buttons != nil){
            for (int i=0;i<buttons.count;i++){
                NSDictionary *obj = [[NSDictionary alloc]init];
                if (buttons[i].label) [obj setValue:buttons[i].label forKey:@"label"];
                if (buttons[i].action) [obj setValue:[self jsonFromPushAction: buttons[i].action] forKey:@"action"];
                if (obj != nil) [array addObject:obj];
            }
        }else{
            return nil;
        }
    } @catch (NSException *exception) {
        NSLog(@"Error jsonArrayFromPushButtons %@", exception);
    }
    return [array copy];
}


- (void) successWithInbox: (Inbox *)inbox newNotifications:(NSArray<INInboxNotification *> *)newNotifications  methodGetInbox:(BOOL)methodGetInbox result: (FlutterResult _Nonnull)result {
    if (methodGetInbox){
        result([self jsonFromInbox: inbox newNotifications:newNotifications callback:CALLBACK_GET_INBOX]);
    }else{
        result([self jsonFromInbox: inbox newNotifications:newNotifications callback:CALLBACK_GET_NEXT_PAGE]);
    }
}

- (void) successWithInboxCounter: (INInboxCounters *)counters  result: (FlutterResult _Nonnull)result {
    result([self jsonFromMessagesCounter: counters]);
    
}

- (void) successWithInboxNotification: (INInboxNotification *)inboxNotification callback: (NSString *)callback result: (FlutterResult _Nonnull)result {
    if (inboxNotification != nil){
        NSArray<INInboxNotification *>* notifications = [NSArray new];
        notifications = [notifications arrayByAddingObject:inboxNotification];
        NSMutableDictionary *dic = [NSMutableDictionary new];
        @try {
            dic = [@{
            callback : [self jsonArrayFromInboxNotification:notifications].firstObject
            } mutableCopy];
        } @catch (NSException *exception) {
            NSLog(@"Error successWithInboxNotification %@", exception);
        }
        result([dic copy]);
    }else{
        [self jsonFromError: [[INError alloc]initWithErrorCode:3020 errorMessage:@"successWithInboxNotification: inboxNotification object is null"] withResult:result];
    }
}

- (void) resultFromString: (NSString *)string withAction: (NSString *)action withResult: (FlutterResult _Nonnull) result {
    NSMutableDictionary *dic = [NSMutableDictionary new];
    @try {
        dic = [@{
        action : string
        } mutableCopy];
    } @catch (NSException *exception) {
        NSLog(@"Error resultFromString %@", exception);
    }
    result([dic copy]);
}


- (void) jsonFromError: (INError *)error withResult: (FlutterResult _Nonnull) result {
    result([FlutterError errorWithCode:[NSString stringWithFormat:@"%u",error.errorCode] message:error.message details:nil]);
}


@end
