import 'dart:convert';
import 'dart:io';

import 'package:indigitall_flutter_plugin/push/models/Push.dart';

import 'InboxCategory.dart';

class InboxNotification {
  /// String field name map id
  static const MAP_ID = "id";

  /// String field name map sent at
  static const MAP_SENT_AT = "sentAt";

  /// String field name status
  static const MAP_STATUS = "status";

  /// String field name sending id
  static const MAP_SENDING_ID = "sendingId";

  /// String field name campaign id
  static const MAP_CAMPAIGN_ID = "campaignId";

  /// String field name push message
  static const MAP_MESSAGE = "message";

  /// String field name read
  static const MAP_READ = "read";

  /// String field name of category
  static const MAP_CATEGORY = "category";

  /// String field name notifications
  static const NOTIFICATIONS = "notifications";

  /// id of notification
  String? id;

  /// notification sent at
  String? sentAt;

  /// status of notification
  String? status;

  /// sendingId of notification
  String? sendingId;

  /// campaign id of notification
  String? campaignId;

  /// push message
  Push? message;

  /// campaign category
  InboxCategory? category;

  /// if is read or not
  bool read = false;

  /// Constructor fo InboxNotification
  ///
  /// Param with Map
  InboxNotification(Map? map) {
    map?.forEach((key, value) {
      if (key == MAP_ID) id = value.toString();
      if (key == MAP_SENT_AT) sentAt = value.toString();
      if (key == MAP_STATUS) status = value.toString();
      if (key == MAP_SENDING_ID) sendingId = value.toString();
      if (key == MAP_MESSAGE) {
        if (Platform.isAndroid) {
          message = new Push(new Map<String, String>.from(json.decode(value)));
        } else {
          //message = new Push(new Map<String, dynamic>.from(value));
        }
      }
      if (key == MAP_CATEGORY) category = new InboxCategory(value);
      if (key == MAP_READ) read = value.toString() == "true";
    });
  }

  /// Create list of notifications from a string
  ///
  /// Param [String] of inbox notifications
  /// return a list of [InboxNotification]
  static List<InboxNotification> createArray(String inboxNotifications) {
    List<dynamic> list = json.decode(inboxNotifications);
    List<InboxNotification> inboxNotificationList = [];
    list.forEach((element) {
      Map map = new Map();
      if (Platform.isAndroid) {
        map = json.decode(element.toString());
      } else {
        map = element;
      }
      InboxNotification inboxNotification = new InboxNotification(map);
      inboxNotificationList.add(inboxNotification);
    });
    return inboxNotificationList;
  }

  /// Conversion to Map
  Map toMap() {
    Map map = Map();
    if (id != null) map[MAP_ID] = id;
    if (sentAt != null) map[MAP_SENT_AT] = sentAt;
    if (status != null) map[MAP_STATUS] = status;
    if (sendingId != null) map[MAP_SENDING_ID] = sendingId;
    if (message != null) map[MAP_MESSAGE] = message;
    map[MAP_READ] = read;

    return map;
  }
}
